import * as tslib_1 from "tslib";
import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./Button.css";
import "./DropdownButton.css";
import * as React from "react";
import { FocusGroupContext } from '../../FocusGroup';
import { FocusZoneContext } from '../../FocusZone';
import { Icon } from '../../Icon';
import { Tooltip } from '../../TooltipEx';
import { css, childCount, getSafeId, KeyCode } from '../../Util';
import { getTabIndex } from '../../Utilities/Focus';
var Button = /** @class */ (function (_super) {
    tslib_1.__extends(Button, _super);
    function Button(props) {
        var _this = _super.call(this, props) || this;
        _this.buttonElement = React.createRef();
        _this.onClick = function (event) {
            if (!_this.props.disabled) {
                var onClick = _this.props.onClick;
                // @NOTE: Safari doesnt set focus to buttons when they are clicked, we need this
                //  to help manage the focus state for callouts.
                _this.focus();
                if (onClick) {
                    onClick(event);
                }
            }
        };
        _this.onFocus = function (event) {
            if (_this.props.onFocus) {
                _this.props.onFocus(event);
            }
            if (_this.focusGroupContext && _this.props.id) {
                _this.focusGroupContext.onFocus(_this.props.id);
            }
        };
        _this.onKeyDown = function (event) {
            if (!event.defaultPrevented && !_this.props.disabled) {
                if (event.which === KeyCode.enter || event.which === KeyCode.space) {
                    if (_this.props.onClick) {
                        _this.props.onClick(event);
                    }
                    event.preventDefault();
                }
            }
        };
        _this.onMouseDown = function (event) {
            if (!event.defaultPrevented) {
                if (_this.props.disabled) {
                    event.preventDefault();
                }
            }
        };
        _this.onMouseLeave = function (event) {
            if (!_this.props.disabled) {
                var onMouseLeave = _this.props.onMouseLeave;
                if (onMouseLeave) {
                    onMouseLeave(event);
                }
            }
        };
        _this.onMouseOver = function (event) {
            if (!_this.props.disabled) {
                var onMouseOver = _this.props.onMouseOver;
                if (onMouseOver) {
                    onMouseOver(event);
                }
            }
        };
        validateProps(props);
        _this.state = {};
        return _this;
    }
    Button.getDerivedStateFromProps = function (props, state) {
        validateProps(props);
        return state;
    };
    Button.prototype.render = function () {
        var _this = this;
        // Determine if the button is an iconOnly button.
        var iconOnly = this.props.iconProps && !this.props.text && childCount(this.props.children) === 0;
        return (React.createElement(FocusGroupContext.Consumer, null, function (focusGroupContext) {
            _this.focusGroupContext = focusGroupContext;
            return (React.createElement(FocusZoneContext.Consumer, null, function (zoneContext) {
                var ButtonType = _this.props.href ? "a" : "button";
                var button = (
                // @ts-ignore TypeScript no longer works with dynamic intrinsic component types.
                React.createElement(ButtonType, { autoFocus: !_this.props.href ? _this.props.autoFocus : undefined, "aria-controls": getSafeId(_this.props.ariaControls), "aria-disabled": _this.props.disabled, "aria-expanded": _this.props.ariaExpanded, "aria-haspopup": _this.props.ariaHasPopup, "aria-label": _this.props.ariaLabel, "aria-setsize": _this.props.ariaSetSize, "aria-posinset": _this.props.ariaPosInSet, "aria-selected": _this.props.ariaSelected, "aria-checked": _this.props.ariaChecked, className: css(_this.props.className, "bolt-button", _this.props.href && "bolt-link-button", _this.props.iconProps && "bolt-icon-button", _this.props.danger && "danger", _this.props.disabled ? "disabled" : "enabled", _this.props.primary && "primary", _this.props.subtle && "subtle", iconOnly && "icon-only", "bolt-focus-treatment"), "data-focuszone": !_this.props.disabled && !_this.props.excludeFocusZone ? zoneContext.focuszoneId : undefined, "data-index": _this.props.dataIndex, "data-is-focusable": !_this.props.excludeFocusZone, href: !_this.props.disabled ? _this.props.href : undefined, id: getSafeId(_this.props.id), onBlur: _this.props.onBlur, onClick: _this.onClick, onMouseLeave: _this.onMouseLeave, onMouseOver: _this.onMouseOver, onKeyDown: _this.onKeyDown, onMouseDown: _this.onMouseDown, onFocus: _this.onFocus, rel: _this.props.rel, role: _this.props.role || "button", tabIndex: getTabIndex(_this.props, focusGroupContext), target: _this.props.target, type: _this.props.type ? _this.props.type : !_this.props.href ? "button" : undefined, ref: _this.buttonElement },
                    _this.props.iconProps &&
                        Icon(Object.assign({}, _this.props.iconProps, {
                            className: css(_this.props.iconProps.className, "left-icon")
                        })),
                    _this.props.text && React.createElement("span", { className: "bolt-button-text" }, _this.props.text),
                    _this.props.children));
                if (_this.props.tooltipProps) {
                    button = (React.createElement(Tooltip, tslib_1.__assign({ showOnFocus: iconOnly }, _this.props.tooltipProps), button));
                }
                return button;
            }));
        }));
    };
    Button.prototype.focus = function () {
        if (this.buttonElement.current) {
            this.buttonElement.current.focus();
        }
    };
    return Button;
}(React.Component));
export { Button };
function validateProps(props) {
    if ($DEBUG) {
        if (props.danger && props.primary) {
            console.warn("primary and danger props are both set to true on Button, only one should be set to true at a time.");
        }
    }
}
