import * as React from "react";
import { IOffset, IOrigin, IPoint } from '../../Utilities/Position';
import { IButtonProps } from "./Button.Props";

export interface IDropdownButton {
    collapse(): void;
    expand(): void;
}

export interface IDropdownButtonProps extends IButtonProps {
    /**
     * Element the dropdown should open relative to, if the dropdown shouldn't
     * open relative to the button itself.
     */
    anchorElement?: HTMLElement;

    /**
     * Offset from the anchor element or point for the dropdown.
     */
    anchorOffset?: IOffset;

    /**
     * Location on the anchor element the dropdown should be relative to.
     */
    anchorOrigin?: IOrigin;

    /**
     * anchorPoint can be supplied if the dropdown should open at a fixed
     * point instead of relative to the button.
     */
    anchorPoint?: IPoint;

    /**
     * Optional Id that should be used on the dropdown to uniquely identify
     * this element.
     */
    dropdownId?: string;

    /**
     * Location on the dropdown to make relative to the button location.
     */
    dropdownOrigin?: IOrigin;

    /**
     * If hideDropdownIcon is supplied the button acts as a dropdown button
     * without showing the dropdown arrows.
     */
    hideDropdownIcon?: boolean;

    /**
     * The ArrowDown key opens the dropdown by default, if ignoreArrowDown
     * is specified the arrow key will not open the dropdown. To open the
     * dropdown the button should be activated.
     */
    ignoreArrowDown?: boolean;

    /**
     * Method used to create the dropdown.
     */
    renderDropdown: (
        dropdown: IDropdownButton,
        dropdownId: string,
        anchorElement?: HTMLElement,
        anchorOffset?: IOffset,
        anchorOrigin?: IOrigin,
        anchorPoint?: IPoint,
        dropdownOrigin?: IOrigin
    ) => JSX.Element;
}