import * as tslib_1 from "tslib";
import "../../FabricInit";
import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./ContextualMenuButton.css";
import * as React from "react";
import { Button } from '../../Button';
import { Icon } from '../../Icon';
import { getId } from '../../Core/Util/Object';
import { css } from '../../Util';
import { Location } from '../../Utilities/Position';
import { VssContextualMenu } from "../../VssContextualMenu";
import { VssIcon } from "../../VssIcon";
import "./ContextualMenuButton.css";
/**
 * Component that contains a button which, when activated, will show a contextual menu with the items provided
 */
var ContextualMenuButton = /** @class */ (function (_super) {
    tslib_1.__extends(ContextualMenuButton, _super);
    function ContextualMenuButton(props) {
        var _this = _super.call(this, props) || this;
        _this._onClick = function (event) {
            var _a = _this.props, onClick = _a.onClick, useTargetPoint = _a.useTargetPoint;
            if (!onClick || !onClick(event)) {
                event.stopPropagation();
            }
            if (!_this.state.isOpen) {
                _this.setState({
                    items: _this.props.getItems ? _this.props.getItems() : []
                });
                if (event.clientX && useTargetPoint) {
                    _this.setState({
                        isOpen: true,
                        providers: _this.props.getItemProviders ? _this.props.getItemProviders() : [],
                        useTargetPoint: true,
                        target: { x: event.clientX, y: event.clientY }
                    });
                }
                else {
                    // if this is a keydown event, we don't want to use target point we want to use target element
                    _this.setState({
                        isOpen: true,
                        providers: _this.props.getItemProviders ? _this.props.getItemProviders() : [],
                        useTargetPoint: false
                    });
                }
            }
            else {
                _this.close();
            }
        };
        _this._onDismiss = function () {
            if (_this._isMounted) {
                if (_this.props.onDismiss) {
                    _this.props.onDismiss();
                }
                _this.close();
            }
        };
        _this.close = function () {
            _this.setState({ target: null, isOpen: false, useTargetPoint: false, items: undefined, providers: undefined }, function () { return _this.focus(); });
        };
        if (_this.props.isOpen) {
            _this.state = {
                isOpen: true,
                providers: _this.props.getItemProviders ? _this.props.getItemProviders() : [],
                items: _this.props.getItems ? _this.props.getItems() : []
            };
        }
        else {
            _this.state = { items: undefined, providers: undefined, isOpen: false };
        }
        _this._contextMenuExpandedId = getId("context-menu-expanded");
        return _this;
    }
    ContextualMenuButton.prototype.focus = function () {
        if (this._button && this._button.focus) {
            this._button.focus();
        }
    };
    ContextualMenuButton.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.isOpen && !this.state.items) {
            this.setState({
                items: this.props.getItems ? this.props.getItems() : [],
                providers: this.props.getItemProviders ? this.props.getItemProviders() : []
            });
        }
    };
    ContextualMenuButton.prototype.componentDidMount = function () {
        this._isMounted = true;
    };
    ContextualMenuButton.prototype.componentWillUnmount = function () {
        this._isMounted = false;
    };
    ContextualMenuButton.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, ctaButton = _a.ctaButton, directionalHint = _a.directionalHint, iconProps = _a.iconProps, title = _a.title, allocateSpaceWhileHidden = _a.allocateSpaceWhileHidden, dataIndex = _a.dataIndex, onFocus = _a.onFocus, tabIndex = _a.tabIndex;
        var isOpen = this.state.isOpen || this.props.isOpen || false;
        var showTooltip = this.props.showTooltip && !isOpen;
        var target = this.state.target || this.props.target || this.props.targetPoint;
        var isTargetAPoint = this.state.useTargetPoint || this.props.useTargetPoint;
        if (isTargetAPoint === undefined) {
            // See target is a point
            isTargetAPoint = target && target.x !== undefined && target.y !== undefined;
        }
        var buttonProps = {
            ariaControls: isOpen ? this._contextMenuExpandedId : null,
            ariaExpanded: isOpen,
            ariaHasPopup: true,
            ariaLabel: title,
            className: css("vss-ContextualMenuButton", "contextual-menu-button", // back compat
            className, isOpen && "open", allocateSpaceWhileHidden && "space-allocated", !allocateSpaceWhileHidden && "space-unallocated"),
            dataIndex: dataIndex,
            excludeTabStop: tabIndex === -1,
            onClick: this._onClick,
            onFocus: onFocus,
            primary: ctaButton
        };
        if (title && showTooltip) {
            buttonProps.tooltipProps = {
                anchorOrigin: { horizontal: Location.center, vertical: Location.end },
                showOnFocus: true,
                text: title,
                tooltipOrigin: { horizontal: Location.center, vertical: Location.start }
            };
        }
        var button = (React.createElement(Button, tslib_1.__assign({ ref: function (button) { return (_this._button = button); } }, buttonProps), iconProps && React.createElement(Icon, tslib_1.__assign({}, VssIcon.getIconProps(iconProps)))));
        return (React.createElement("div", { className: ContextualMenuButton.ButtonContainerClassName, ref: ContextualMenuButton.buttonRef },
            button,
            isOpen && this.state.items && (React.createElement(VssContextualMenu, { providers: this.state.providers, items: this.state.items, onDismiss: this._onDismiss, shouldFocusOnMount: true, gapSpace: isTargetAPoint ? 5 : 0, directionalHint: directionalHint, target: target || this.refs[ContextualMenuButton.buttonRef], ariaLabel: title }))));
    };
    ContextualMenuButton.ButtonContainerClassName = "vss-ContextualMenu--container";
    ContextualMenuButton.buttonRef = "button";
    return ContextualMenuButton;
}(React.Component));
export { ContextualMenuButton };
