export interface IDateTimeRangePickerProps {
    /**
     * Initial value for the start time, for an uncontrolled instance of the DateTimePicker for StartTime.
     */
    defaultStartTime?: Date;

    /**
     * Initial value for the end time, for an uncontrolled instance of the DateTimePicker for EndTime.
     */
    defaultEndTime?: Date;

    /**
     * Optional method to format the chosen date to a string to display in the DatePicker
     */
    formatDate?: (date: Date) => string;

    /**
     * Optional method to format the chosen time to a string to display in the TextField
     */
    formatTime?: (date: Date) => string;

    /**
     * Optional method to parse the text input value to time
     */
    parseTimeFromString?: (timeStr: string) => Date | null;

    /**
     * Callback issued when either date/time is changed legally
     */
    onValueChanged?: (newStartTime: Date, newEndTime: Date) => void;

    /**
     * Earliest pickable date.
     */
    minDate?: Date;

    /**
     * Latest pickable date.
     */
    maxDate?: Date;

    /**
     * Optional aria label for start date text field. If not provided, default will be used.
     */
    startDateAriaLabel?: string;
    /**
     * Optional aria label for start time text field. If not provided, default will be used.
     */
    startTimeAriaLabel?: string;
    /**
     * Optional aria label for end date text field. If not provided, default will be used.
     */
    endDateArialLabel?: string;
    /**
     * Optional aria label for end time text field. If not provided, default will be used.
     */
    endTimeAriaLabel?: string;
}