import * as React from "react";
import { KeyCode } from '../../Util';
import { ITooltipProps } from '../../TooltipEx';
export interface IExpandableProps {
    /**
     * css class that is added to the dropdown element.
     */
    className?: string;

    /**
     * The collapseDelay is used in conjunction with the expandOnHover to determine
     * how long the mouse must be out of the expandable to collapse. This only applies
     * if the expandable was expanded due to hover. The collapseDelay is defined in
     * milliseconds.
     *
     * @default 250
     */
    collapseDelay?: number;

    /**
     * If true, the button cannot be interacted with.
     */
    disabled?: boolean;

    /**
     * Set to true if you don't want the dropdown's focus managed by a FocusZone.
     */
    excludeFocusZone?: boolean;

    /**
     * Set to true if you don't want the dropdown's to be tabbable.
     */
    excludeTabStop?: boolean;

    /**
     * Id that is used in the aria-controls attribute. This should line up with
     * the id rendered into the expandable when a callout is rendered.
     */
    expandableId?: string;

    /**
     * The expandDelay is used in conjunction with the expandOnHover to determine
     * how long the mouse must be over the expandable to expand. The expandDelay is
     * defined in milliseconds.
     *
     * @default 250
     */
    expandDelay?: number;

    /**
     * The key used to expand the components callout.
     *
     * @default KeyCode.DownArrow
     */
    expandKey?: KeyCode | KeyCode[];

    /**
     * If expandOnHover controls whether or not the expandable element is shown
     * when the mouse hovers over the element.
     *
     * @default false
     */
    expandOnHover?: boolean;

    /**
     * id that will be placed on the expandable id element.
     */
    id?: string;

    /**
     * Optional aria role.
     */
    role?: string;

    /**
     * Sets the tabindex on the dropdown element.
     */
    tabIndex?: number;

    /**
     * renderCallout is called when the component is expanded. This should return
     * the elements needed to render the appropriate callout.
     */
    renderCallout: (expandable: IExpandable, id: string, element: HTMLElement) => JSX.Element;
}

/**
 * An IExpandable component supports an expande and collapsed state.
 */
export interface IExpandable {
    /**
     * Collapsing the component will remove any callout's that are rendered when the
     * component is expanded.
     */
    collapse: () => void;

    /**
     * Expanding the component will render any callout's assocaited with the expandable
     * component.
     */
    expand: () => void;
}