import { IMenuItem } from '../../Menu';
import { ITooltipProps } from '../../TooltipEx';
import { IFilter, FILTER_CHANGE_EVENT } from '../../Utilities/Filter';
import { IObservableValue } from '../../Core/Observable';
export interface IHeaderCommandBar {}

export interface IHeaderCommandBarItem extends IMenuItem {
    /**
     * An aria label to add to the command bar item.
     */
    ariaLabel?: string;

    /**
     * A value of true means the item will always be rendered as
     * a button directly in the header.
     * A value of false means the item will always be rendered in
     * an overflow button.
     * Undefined means that the HeaderCommandBar will determine
     * where the item is rendered.
     */
    important?: boolean;

    /**
     * Determines if we render as a primary button or not
     */
    isPrimary?: boolean;

    /**
     * Uses a subtle button when rendering this item - only applies
     * when the item is not in overflow
     */
    subtle?: boolean;

    /**
     * Optional tooltip props to display
     */
    tooltipProps?: ITooltipProps;
}

export interface IHeaderCommandBarProps {
    /**
     * Additional classes to be applied to component
     */
    className?: string;

    /**
     * Items to display in the far-right Command Bar
     * Should influence page-wide state
     */
    items: IHeaderCommandBarItem[];
}

export interface IHeaderCommandBarWithFilterProps extends IHeaderCommandBarProps {
    /**
     * A filter object to listen to changes on. When there are changes, the filter icon
     * will be displayed as filled.
     */
    filter: IFilter;

    /**
     * The current state of whether the filter is visible or hidden. The value is toggled
     * by the onClick on the item.
     */
    filterToggled: IObservableValue<boolean | undefined>;
}