import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./List.css";
import "./ListDropIndicator.css";
import * as React from "react";
import { IObservableArray, IReadonlyObservableValue } from '../../Core/Observable';
import { IEventDispatch } from '../../Utilities/Dispatch';
import { IItemProvider } from '../../Utilities/Provider';
import { IListCellDetails, IList, IOverlayRenderProps, ISimpleListCell, IListItemDetails, IListItemProps, IListProps, ISimpleListProps } from "./List.Props";
interface IRowOverlay {
    id: string;
    render: (props: IOverlayRenderProps) => React.ReactNode;
    rowIndex: number;
    zIndex: number;
}
export interface IListState<T> {
    columns: number;
    eventDispatch: IEventDispatch;
    firstRow: number;
    itemProvider: IItemProvider<T>;
    lastRow: number;
    overlays: IObservableArray<IRowOverlay>;
    pageSize: number;
    renderedRows: {
        [rowIndex: number]: JSX.Element | null;
    };
    rowCount: number;
    rowHeight: number;
    rows: {
        [rowIndex: number]: T | IReadonlyObservableValue<T | undefined>;
    };
    scrollTop: number;
}
/**
 * The List component is used to render a collection of items with a series of rows.
 */
export declare class List<T> extends React.Component<IListProps<T>, IListState<T>> implements IList<T> {
    static defaultProps: Partial<IListProps<any>>;
    static getDerivedStateFromProps<T>(props: Readonly<IListProps<T>>, state: Readonly<IListState<T>>): Partial<IListState<T>> | null;
    private intersectionContext;
    private listElement;
    private bodyElement;
    private spacerElements;
    private focusIndex;
    constructor(props: Readonly<IListProps<T>>);
    render(): JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
    addOverlay(id: string, rowIndex: number, render: (props: IOverlayRenderProps) => React.ReactNode, zIndex?: number): void;
    removeOverlay(id: string): void;
    getFocusIndex(): number;
    private clearCache;
    private onDispatch;
    private onClick;
    private onDoubleClick;
    private onFocusItem;
    private onKeyDown;
    private onIntersect;
    private renderLoadingRow;
    private renderOverlay;
    private renderRow;
    private renderSpacer;
    private rowActivated;
    private rowSelected;
}
export declare class ScrollableList<T> extends React.Component<IListProps<T>> implements IList<T> {
    private list;
    render(): JSX.Element;
    addOverlay(id: string, rowIndex: number, render: (props: IOverlayRenderProps) => React.ReactNode, zIndex?: number): void;
    removeOverlay(id: string): void;
    getFocusIndex(): number;
}
export declare class SimpleList extends React.Component<ISimpleListProps> {
    render(): JSX.Element;
    private renderListItem;
}
export declare function renderListItem<T>(rowIndex: number, details: IListItemDetails<T>, children: JSX.Element): JSX.Element;
export declare class ListItem<T> extends React.Component<IListItemProps<T>> {
    render(): JSX.Element;
    private onFocus;
}
export declare function renderListCell(listCell: ISimpleListCell | string | number): JSX.Element;
export declare function cellFromEvent(event: React.SyntheticEvent<HTMLElement>): IListCellDetails;
export {};
