import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./List.css";
import "./ListDropIndicator.css";
import * as React from "react";
import { beginDragOperation, DragDropEffect } from '../../Utilities/DragDrop';
import { cellFromEvent } from "./List";
/**
 * A behavior that turns a list into the source of a drag and drop operation. Note - this
 * behavior should only be used if your list is intended to _only_ be a drag source. Please use
 * the ListDragDropBehavior instead if the list is also a drop target, as that is the only way
 * to get keyboard drag and drop support within your list.
 */
var ListDragSourceBehavior = /** @class */ (function () {
    function ListDragSourceBehavior(options) {
        var _this = this;
        this.initialize = function (props, dragDroppableUI, eventDispatch) {
            _this.dragDroppableUI = dragDroppableUI;
            _this.eventDispatch = eventDispatch;
            _this.eventDispatch.addEventListener("mousedown", _this.onMouseDown);
            _this.eventDispatch.addEventListener("touchstart", _this.onTouchStart);
            _this.eventDispatch.addEventListener("dragstart", _this.onDragStart);
            _this.eventDispatch.addEventListener("dragend", _this.onDragEnd);
            _this.itemProvider = props.itemProvider;
        };
        this.onDragEnd = function (event) {
            var index = cellFromEvent(event).rowIndex;
            if (index >= 0 && _this.options.onDragEnd) {
                _this.options.onDragEnd(event);
            }
            _this.dragDroppableUI.removeOverlay("drag-source-item");
        };
        this.onDragStart = function (event) {
            var index = cellFromEvent(event).rowIndex;
            if (index >= 0) {
                if (_this.options.onDragStart) {
                    _this.options.onDragStart(event);
                }
                if (event.detail.dataTransfer.effectAllowed !== DragDropEffect.none) {
                    _this.dragDroppableUI.addOverlay("drag-source-item", index, _this.renderDragSourceItemOverlay);
                }
            }
        };
        this.onMouseDown = function (event) {
            _this.beginDrag(event);
        };
        this.onTouchStart = function (event) {
            _this.beginDrag(event);
        };
        this.renderDragSourceItemOverlay = function (props) {
            return React.createElement("div", { className: "bolt-list-drag-source-item flex-grow" });
        };
        this.options = options;
    }
    ListDragSourceBehavior.prototype.componentDidUpdate = function (props) {
        this.itemProvider = props.itemProvider;
    };
    ListDragSourceBehavior.prototype.componentWillUnmount = function () {
        this.eventDispatch.removeEventListener("mousedown", this.onMouseDown);
        this.eventDispatch.removeEventListener("dragstart", this.onDragStart);
        this.eventDispatch.removeEventListener("dragend", this.onDragEnd);
    };
    ListDragSourceBehavior.prototype.beginDrag = function (event) {
        var index = cellFromEvent(event).rowIndex;
        if (this.itemProvider && index >= 0) {
            var item = this.itemProvider.value[index];
            beginDragOperation(event, {
                data: item,
                type: this.options.type,
                secondaryData: { index: index, sourceId: this.options.id },
                dropEffect: DragDropEffect.none
            });
        }
    };
    return ListDragSourceBehavior;
}());
export { ListDragSourceBehavior };
