import * as tslib_1 from "tslib";
import { Selection } from '../../Utilities/Selection';
import { KeyCode } from '../../Util';
import { cellFromEvent } from "./List";
var ListSelection = /** @class */ (function (_super) {
    tslib_1.__extends(ListSelection, _super);
    function ListSelection(options) {
        var _this = _super.call(this, typeof options === "boolean" || options === undefined
            ? options
            : { multiSelect: options.multiSelect, lockedRanges: options.lockedRanges, selectedRanges: options.selectedRanges }) || this;
        _this.mousedownPreFocus = false;
        _this.pivotIndex = -1;
        _this.arrowKey = false;
        _this.shiftKey = false;
        _this.metaKey = false;
        _this.ctrlKey = false;
        _this.selectOnFocus = true;
        _this.initialize = function (hasSelection, selectableUI, eventDispatch) {
            _this.selectableUI = selectableUI;
            // Signup for the events we are tracking on the list if a selection is available.
            eventDispatch.addEventListener("click", _this.onClick);
            eventDispatch.addEventListener("keydown", _this.onKeyDown);
            eventDispatch.addEventListener("keyup", _this.onKeyUp);
            eventDispatch.addEventListener("mousedown", _this.onMouseDown);
            eventDispatch.addEventListener("select", _this.onSelect);
        };
        _this.onClick = function (event) {
            if (_this.mousedownPreFocus) {
                return;
            }
            if (!event.defaultPrevented) {
                var index = cellFromEvent(event).rowIndex;
                if (index >= 0) {
                    if (!_this.ctrlKey && !_this.metaKey && !_this.multiSelect) {
                        _this.select(index, 1, false);
                    }
                    else {
                        _this.toggleSelection(index, true);
                    }
                }
            }
        };
        _this.onMouseDown = function (event) {
            var selectableUI = _this.selectableUI;
            // If this mousedown event is happening on the row before it gets focus
            // we need to ignore the click event since focus will update the selection state.
            _this.mousedownPreFocus = cellFromEvent(event).rowIndex !== selectableUI.getFocusIndex() && !!_this.selectOnFocus;
            // We need to udpate the meta key states on mousedown since a user may be clicking
            // into the list with a set of keys pressed.
            _this.shiftKey = event.shiftKey;
            _this.ctrlKey = event.ctrlKey;
            _this.metaKey = event.metaKey;
        };
        _this.onKeyDown = function (event) {
            // Track the state of the keys we use to effect the selection types.
            if (event.which === KeyCode.upArrow || event.which === KeyCode.downArrow) {
                _this.arrowKey = true;
            }
            if (event.which === KeyCode.shift) {
                _this.shiftKey = true;
            }
            else if (event.which === KeyCode.ctrl) {
                _this.ctrlKey = true;
            }
            else if (event.which === KeyCode.macCommand) {
                _this.metaKey = true;
            }
            if (!event.defaultPrevented) {
                // Toggle the row selection
                if (event.which === KeyCode.space || event.which === KeyCode.enter) {
                    var focusIndex = _this.selectableUI.getFocusIndex();
                    if (focusIndex >= 0 && _this.indexSelectable(focusIndex)) {
                        _this.toggleSelection(focusIndex, true);
                        _this.pivotIndex = focusIndex;
                    }
                }
            }
        };
        _this.onKeyUp = function (event) {
            // Track the state of the keys we use to effect the selection types.
            if (event.which === KeyCode.upArrow || event.which === KeyCode.downArrow) {
                _this.arrowKey = false;
            }
            else if (event.which === KeyCode.shift) {
                _this.shiftKey = false;
            }
            else if (event.which === KeyCode.ctrl) {
                _this.ctrlKey = false;
            }
            else if (event.which === KeyCode.macCommand) {
                _this.metaKey = false;
            }
        };
        _this.onSelect = function (event, indexedValue) {
            if (!_this.selectOnFocus) {
                return;
            }
            // Update selection based on the focused row.
            var index = indexedValue.index;
            if (index >= 0) {
                // If a selection occurs during an arrow key with shift we will not
                // udpate the selection state on the row select event.
                if (_this.shiftKey || (!_this.ctrlKey && !_this.metaKey) || !_this.arrowKey) {
                    if (_this.shiftKey && _this.pivotIndex >= 0 && _this.multiSelect) {
                        _this.select(Math.min(_this.pivotIndex, index), Math.abs(_this.pivotIndex - index) + 1, _this.ctrlKey || _this.metaKey);
                    }
                    else {
                        if ((_this.ctrlKey || _this.metaKey) && _this.multiSelect) {
                            _this.toggleSelection(index, true);
                        }
                        else {
                            _this.select(index, 1, false);
                        }
                    }
                    // Save the last selectionIndex that we selected, this will allow
                    // us to perform range based selection.
                    if (!_this.shiftKey) {
                        _this.pivotIndex = index;
                    }
                }
            }
        };
        if (typeof options !== "boolean" && options !== undefined) {
            _this.selectOnFocus = options.selectOnFocus || false;
        }
        return _this;
    }
    ListSelection.prototype.toggleSelection = function (index, merge) {
        if (this.selected(index)) {
            this.unselect(index);
        }
        else {
            this.select(index, 1, merge);
        }
    };
    return ListSelection;
}(Selection));
export { ListSelection };
