/// <reference types="react" />
import { IListSelection } from '../../List';
import { IIconProps } from '../../Icon';
import { ITableColumn } from '../../Table';
import { IFocusZoneProps } from '../../FocusZone';
import { IReadonlyObservableArray } from '../../Core/Observable';
export interface IListBoxProps<T> {
    /**
     * The items to render within the list box.
     */
    items: IListBoxItem<T>[] | IReadonlyObservableArray<IListBoxItem<T>>;
    /**
     * focuszoneProps allows the caller to manage the how the list box rows are focused.
     * The default focuszone if one isnt supplied is a Vertical non-cyclic focus zone.
     */
    focuszoneProps?: IFocusZoneProps;
    /**
     * onSelect is called when the row is selected. Selection occurs on the
     * Space keystorke or click.
     *
     * @param event - This is the event that is causing the selection.
     * @param listRow - Details about the list row being selected.
     */
    onSelect?: (event: React.SyntheticEvent<HTMLElement>, item: IListBoxItem<T>) => void;
    /**
     * The selection maintains the selected state and defines the selection behavior of the list box.
     */
    selection: IListSelection;
}
export interface IListBoxItem<T> {
    /**
     * className to pass to the item's cell.
     */
    className?: string;
    /**
     * Optional item data to be used in custom render function.
     */
    data?: T;
    /**
     * Props specifying the icon to render beside the item text.
     */
    iconProps?: IIconProps;
    /**
     * Unique identifier of the item.
     */
    id: string;
    /**
     * Text to render in the item's row.
     */
    text?: string;
    /**
     * A function to render custom item content.
     */
    render?: (rowIndex: number, columnIndex: number, tableColumn: ITableColumn<IListBoxItem<T>>, tableItem: IListBoxItem<T>) => JSX.Element;
    /**
     * The type of list item.  Headers and Separators ignore focus and selection.
     */
    type?: ListBoxItemType;
}
export declare enum ListBoxItemType {
    Row = 1,
    Header = 2,
    Divider = 3
}
