import * as tslib_1 from "tslib";
import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./ListBox.css";
import * as React from "react";
import { ListBoxItemType } from "./ListBox.Props";
import { ArrayItemProvider } from '../../Utilities/Provider';
import { Table, SimpleTableCell, TableRow, ColumnSelect, renderEmptyCell } from '../../Table';
import { renderListCell } from '../../List';
import { Observer } from '../../Observer';
import { css } from '../../Util';
import { ObservableLike } from '../../Core/Observable';
var ListBox = /** @class */ (function (_super) {
    tslib_1.__extends(ListBox, _super);
    function ListBox(props) {
        var _this = _super.call(this, props) || this;
        _this.tabbableIndex = -1;
        _this.positions = [];
        _this.count = 0;
        _this.onItemsChanged = function () {
            _this.tabbableIndex = -1;
            _this.positions = [];
            _this.count = 0;
            for (var _i = 0, _a = ObservableLike.getValue(_this.props.items); _i < _a.length; _i++) {
                var item = _a[_i];
                if (item.type === ListBoxItemType.Divider || item.type === ListBoxItemType.Header) {
                    _this.positions.push(-1);
                }
                else {
                    if (_this.tabbableIndex === -1 && _this.props.selection.indexSelectable(_this.positions.length)) {
                        _this.tabbableIndex = _this.positions.length;
                    }
                    _this.positions.push(++_this.count);
                }
            }
            return true;
        };
        _this.onSelect = function (event, tableRow) {
            var items = ObservableLike.getValue(_this.props.items);
            if (_this.props.onSelect && items) {
                _this.props.onSelect(event, items[tableRow.index]);
            }
        };
        _this.renderListBoxRow = function (index, item, details) {
            var selection = _this.props.selection;
            var focusable = _this.props.selection.indexSelectable(index);
            var rowDetails = tslib_1.__assign({}, details, { ariaPosInSet: _this.positions[index] >= 0 ? _this.positions[index] : null, ariaSetSize: _this.positions[index] >= 0 ? _this.count : null, excludeTabStop: _this.tabbableIndex !== index, excludeFocusZone: !focusable });
            return (React.createElement(Observer, { key: index, item: item }, function () { return (React.createElement(TableRow, { key: index, index: index, details: rowDetails, className: css(item.type === ListBoxItemType.Header && "bolt-list-box-header-row", item.type === ListBoxItemType.Divider && "bolt-list-box-divider-row", selection.multiSelect && "bolt-list-box-multiselect-row") }, _this.columns.map(function (tableColumn, columnIndex) {
                if (_this.props.selection.multiSelect && columnIndex === 0 && !focusable) {
                    return renderEmptyCell(index, columnIndex);
                }
                return tableColumn.renderCell(index, columnIndex, tableColumn, item);
            }))); }));
        };
        _this.renderListBoxCell = function (rowIndex, columnIndex, tableColumn, tableItem) {
            if (tableItem.render) {
                return tableItem.render(rowIndex, columnIndex, tableColumn, tableItem);
            }
            if (tableItem.type === ListBoxItemType.Divider) {
                return (React.createElement(SimpleTableCell, { className: css(tableColumn.className, tableItem.className), columnIndex: columnIndex, key: columnIndex, tableColumn: tableColumn },
                    React.createElement("div", { className: "bolt-list-box-divider flex-grow" })));
            }
            return (React.createElement(SimpleTableCell, { className: css(tableColumn.className, tableItem.className, tableItem.type === ListBoxItemType.Header && "bolt-list-box-header"), columnIndex: columnIndex, key: columnIndex, tableColumn: tableColumn }, tableItem && renderListCell(tableItem)));
        };
        var selection = _this.props.selection;
        _this.columns = selection.multiSelect ? [new ColumnSelect()] : [];
        _this.columns.push({
            id: "text",
            width: 200,
            renderCell: _this.renderListBoxCell,
            className: css("bolt-list-box-text", !selection.multiSelect && "bolt-list-box-text-single-select")
        });
        _this.onItemsChanged();
        return _this;
    }
    ListBox.prototype.render = function () {
        var _this = this;
        var _a = this.props, focuszoneProps = _a.focuszoneProps, selection = _a.selection;
        var itemsObservable = { observableValue: this.props.items, filter: this.onItemsChanged };
        return (React.createElement(Observer, { items: itemsObservable }, function (props) {
            var tableItemProvider = new ArrayItemProvider(props.items);
            return (React.createElement(Table, { className: "bolt-list-box", columns: _this.columns, focuszoneProps: focuszoneProps, itemProvider: tableItemProvider, onSelect: _this.onSelect, renderRow: _this.renderListBoxRow, scrollable: true, selection: selection, showHeader: false, showLines: false, spacerWidth: 0 }));
        }));
    };
    return ListBox;
}(React.Component));
export { ListBox };
export function getNonSelectableRanges(items, itemSelectable) {
    if (itemSelectable === void 0) { itemSelectable = listBoxItemSelectable; }
    var ranges = [];
    var beginIndex = -1;
    for (var index = 0; index < items.length; index++) {
        if (!itemSelectable(items[index]) && beginIndex < 0) {
            beginIndex = index;
        }
        else if (itemSelectable(items[index]) && beginIndex >= 0) {
            ranges.push({ beginIndex: beginIndex, endIndex: index - 1 });
            beginIndex = -1;
        }
    }
    if (beginIndex >= 0) {
        ranges.push({ beginIndex: beginIndex, endIndex: items.length - 1 });
    }
    return ranges;
}
export function listBoxItemSelectable(item) {
    return item.type !== ListBoxItemType.Header && item.type !== ListBoxItemType.Divider;
}
