import "../../FabricInit";
import { ISuggestionModel as FabricSuggestionModel } from "office-ui-fabric-react/lib/Pickers";
import { ISharedPeoplePickerProps, IIdentity } from "../PeoplePicker/SharedPeoplePicker.Props";

export interface ISuggestionModel extends FabricSuggestionModel<IIdentity> {}

export interface IPeoplePickerDropdown {
    hidePicker: () => void;
    showPicker: () => void;
    updateSuggestionsList: (suggestions: IIdentity[] | PromiseLike<IIdentity[]>, updatedValue?: string) => void;
    currentSuggestion: () => IIdentity | undefined;
}

export interface IPeoplePickerDropdownProps extends ISharedPeoplePickerProps {
    /**
     * Aria label
     * @default "Search users and groups"
     */
    ariaLabel?: string;

    /**
     * A boolean value indicating should component gets auto focussed on mount
     */
    autoFocus?: boolean;

    /**
     * Width of callout
     * @default 300.
     */
    calloutWidth?: number;

    /**
     * The CSS classname of the people picker control.
     */
    className?: string;

    /**
     * Gets the component ref.
     */
    componentRef?: (ref: IPeoplePickerDropdown | null) => void;

    /**
     * Default Identity to initially render in the dropdown.
     */
    defaultIdentity?: IIdentity;

    /**
     * Indicates if the text in resultsFooter or resultsFooterFull should be shown at the end of the suggestion list.
     * @default true.
     */
    isResultsFooterVisible?: boolean;

    /**
     * A callback for when input gets blurred
     */
    onBlur?: () => void;

    /**
     * A callback for when a suggestion is clicked. If no valid suggestion has been selected, item is undefined.
     */
    onChange?: (item?: IIdentity) => void;

    /**
     * A callback for when the search text is cleared.
     */
    onClear?: () => void;

    /**
     * A callback for when input gets focus
     */
    onFocus?: (e: React.FocusEvent<HTMLInputElement>) => void;

    /**
     * Maximum number of suggestions to show in the full suggestion list.
     */
    resultsMaximumNumber?: number;

    /**
     * An ARIA label for the container that is the parent of the suggestions.
     */
    suggestionsContainerAriaLabel?: string;

    /**
     * the classname of the suggestionitem.
     */
    suggestionsItemClassName?: string;
}