import * as tslib_1 from "tslib";
import "../../FabricInit";
import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./PickList.css";
import "./PickListDropdown.css";
import * as React from "react";
import { Callout } from "office-ui-fabric-react/lib/Callout";
import { BaseComponent } from "office-ui-fabric-react/lib/Utilities";
import { getId } from '../../Core/Util/Object';
import { FocusZoneContext } from '../../FocusZone';
import { css, getSafeId, KeyCode } from '../../Util';
import { format } from '../../Core/Util/String';
import { SelectionMode } from "office-ui-fabric-react/lib/Selection";
import { Tooltip } from '../../TooltipEx';
import { PickList } from "../../Components/PickList/PickList";
import { VssIcon, VssIconType } from '../../VssIcon';
import { ItemIndicatorSet } from '../../ItemIndicator';
import "./PickListDropdown.css";
var DEFAULT_PICKLIST_WIDTH = 200;
var DEFAULT_VIRTUALIZATION_LIMIT = 50;
var PickListDropdown = /** @class */ (function (_super) {
    tslib_1.__extends(PickListDropdown, _super);
    function PickListDropdown(props) {
        var _this = _super.call(this, props) || this;
        _this._onBlur = function () {
            if (!_this._getEffectiveIsListOpen()) {
                if (_this.props.onFocusLost) {
                    _this.props.onFocusLost();
                }
            }
        };
        _this._onClick = function () {
            if (!_this.props.disabled) {
                _this._togglePickList(!_this._getEffectiveIsListOpen());
            }
        };
        _this._onKeyDown = function (ev) {
            if (!_this.props.disabled) {
                if (ev.which === KeyCode.enter || ev.which === KeyCode.downArrow) {
                    _this._togglePickList(!_this._getEffectiveIsListOpen());
                    ev.preventDefault();
                }
                else if (ev.which === KeyCode.escape) {
                    _this._togglePickList(false);
                    _this._dropdownTitle.focus();
                }
            }
        };
        _this._onPickListFocusLost = function () {
            _this._shouldRefocus = true;
            _this._togglePickList(false);
        };
        _this._onPositioned = function () {
            if (_this._shouldRefocus) {
                _this._shouldRefocus = false;
                if (_this._pickList) {
                    _this._pickList.focus();
                }
            }
        };
        _this._onDismiss = function (ev) {
            _this._shouldRefocus = true;
            _this._togglePickList(false);
            if (ev.which === KeyCode.escape) {
                _this._dropdownTitle.focus();
            }
        };
        _this._onSelectionChanged = function (selection) {
            var isListOpen = _this._getEffectiveIsListOpen() && _this.props.selectionMode === SelectionMode.multiple;
            _this._togglePickList(isListOpen);
            if (_this._maintainsOwnState) {
                _this.setState({
                    selectedItems: selection.selectedItems
                });
            }
            if (_this.props.onSelectionChanged) {
                return _this.props.onSelectionChanged(selection);
            }
        };
        _this._getNameFromItem = function (item) {
            if (_this.props.getListItem) {
                return _this.props.getListItem(item).name;
            }
            return item;
        };
        _this._getIconFromItem = function (item) {
            if (_this.props.getListItem) {
                return _this.props.getListItem(item).iconProps;
            }
            return undefined;
        };
        _this._id = getId("PickListDropdown");
        _this._maintainsOwnState = false;
        if (props.initiallySelectedItems && props.selectedItems) {
            console.warn("Please only provide ONE of initiallySelectedItems OR selectedItems as props");
        }
        if (props.initiallySelectedItems) {
            _this._maintainsOwnState = true;
        }
        _this.state = {
            isListOpen: false,
            pickListWidth: props.pickListWidth || props.width || DEFAULT_PICKLIST_WIDTH,
            selectedItems: _this._maintainsOwnState ? props.initiallySelectedItems : undefined
        };
        _this._shouldRefocus = true;
        return _this;
    }
    PickListDropdown.prototype.focus = function () {
        if (this._getEffectiveIsListOpen()) {
            this._pickList.focus();
        }
        else {
            this._dropdownTitle.focus();
        }
    };
    PickListDropdown.prototype.forceUpdate = function () {
        if (this._pickList) {
            this._pickList.forceUpdate();
        }
    };
    PickListDropdown.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, pickListClassName = _a.pickListClassName, placeholder = _a.placeholder, getPickListItems = _a.getPickListItems, disabled = _a.disabled, selectionMode = _a.selectionMode, _b = _a.preventDismissOnScroll, preventDismissOnScroll = _b === void 0 ? true : _b, tabIndex = _a.tabIndex, hideSelectedItemIcon = _a.hideSelectedItemIcon, _c = _a.dropdownIconProps, dropdownIconProps = _c === void 0 ? {
            iconType: VssIconType.fabric,
            iconName: "ChevronDown"
        } : _c, directionalHint = _a.directionalHint, onRenderSelectedItemText = _a.onRenderSelectedItemText, id = _a.id, onFocus = _a.onFocus, excludeFocusZone = _a.excludeFocusZone;
        var selectedItems = this._maintainsOwnState ? this.state.selectedItems : this.props.selectedItems;
        var isListOpen = this._getEffectiveIsListOpen();
        var hasSelectedItems = selectedItems && selectedItems.length > 0;
        var titleText = hasSelectedItems ? this._getTitleText(selectedItems) : placeholder;
        var fullClassName = css("vss-PickListDropdown", className);
        var fullPickListClassName = css("vss-PickListDropdown--list", pickListClassName);
        var placeholderLabel;
        if (this.props.showPlaceholderAsLabel && placeholder && hasSelectedItems) {
            placeholderLabel = React.createElement("span", { className: "vss-PickListDropdown--title-text vss-PickListDropdown--placeholder-label" }, placeholder + ": ");
        }
        var ariaLabel = placeholderLabel ? placeholder + ": " + titleText : titleText;
        if (this.props.ariaLabelFormat) {
            ariaLabel = format(this.props.ariaLabelFormat, hasSelectedItems ? titleText : "");
        }
        if (isListOpen) {
            this._shouldRefocus = true;
        }
        var iconProps = undefined;
        if (hasSelectedItems && !hideSelectedItemIcon) {
            if (selectionMode === SelectionMode.multiple) {
                iconProps = {
                    iconType: VssIconType.fabric,
                    iconName: "CheckMark",
                    className: "vss-PickListDropdown--title-checkIcon"
                };
            }
            else {
                iconProps = this._getIconFromItem(selectedItems[0]);
            }
        }
        var pickListId = id || getId("PickList");
        var style = {};
        if (this.props.width) {
            style.width = this.props.width + "px";
        }
        return (React.createElement("div", { className: fullClassName, ref: this._resolveRef("_dropdown"), style: style },
            React.createElement(FocusZoneContext.Consumer, null, function (zoneContext) {
                return (React.createElement("div", { tabIndex: disabled ? -1 : tabIndex || 0, ref: function (element) { return (_this._dropdownTitle = element); }, onClick: _this._onClick, onKeyDown: _this._onKeyDown, className: css("vss-PickListDropdown--title", !hasSelectedItems && "vss-PickListDropdown--placeholder", disabled && "disabled", isListOpen && "focused"), "aria-expanded": isListOpen, "aria-label": ariaLabel, "aria-describedby": getSafeId(_this.props.ariaDescribedBy), "aria-disabled": !!disabled, "data-focuszone": !disabled && !excludeFocusZone && zoneContext.focuszoneId, role: "button", id: getSafeId(_this._id), "aria-owns": isListOpen ? getSafeId(pickListId) : null, onFocus: onFocus, onBlur: _this._onBlur },
                    iconProps && React.createElement(VssIcon, tslib_1.__assign({}, iconProps)),
                    React.createElement(Tooltip, { overflowOnly: true, text: titleText },
                        React.createElement("div", { className: "vss-PickListDropdown--title-textContainer" },
                            placeholderLabel,
                            React.createElement("span", { className: css("vss-PickListDropdown--title-text", hasSelectedItems && "vss-PickListDropdown--title-text-withSelections"), role: "textbox", "aria-readonly": true }, onRenderSelectedItemText ? onRenderSelectedItemText(selectedItems) : React.createElement(React.Fragment, null, titleText)))),
                    React.createElement(VssIcon, tslib_1.__assign({}, dropdownIconProps, { className: css(dropdownIconProps.className, "vss-PickListDropdown--title-caretIcon") })),
                    _this.getIndicators(selectedItems)));
            }),
            isListOpen && (React.createElement(Callout, { className: "vss-PickListDropdown--callout", onDismiss: this._onDismiss, onPositioned: this._onPositioned, target: this._dropdown, isBeakVisible: false, preventDismissOnScroll: preventDismissOnScroll, gapSpace: 0, directionalHint: directionalHint !== undefined ? directionalHint : 4 /* bottomLeftEdge */ },
                React.createElement(PickList, tslib_1.__assign({}, this.props, { initiallySelectedItems: undefined, selectedItems: selectedItems, width: this.state.pickListWidth, className: fullPickListClassName, onSelectionChanged: this._onSelectionChanged, ref: function (pickList) { return (_this._pickList = pickList); }, items: getPickListItems(), onGroupLoadRequest: this.props.onGroupLoadRequest, onFocusLost: this._onPickListFocusLost, id: pickListId, shouldVirtualize: function (items) {
                        // Virtualization is disabled for the number of items below certain limit since callout positioning happens
                        // before new pages are added which ends up a wrong callout position.
                        //
                        // Scenario is the space below the target may be sufficient for the first page and callout is
                        // positioned below target. Once the other pages are added, callout needs to be repositioned
                        // to be next to or even on top of target which does not happen.
                        //
                        // By specifying a high limit we make sure that positioning will take as much space as possible so that
                        // no repositioning is required.
                        return (items || []).length > DEFAULT_VIRTUALIZATION_LIMIT;
                    } }))))));
    };
    PickListDropdown.prototype.componentDidMount = function () {
        if (this.props.autoFocus) {
            this.focus();
        }
    };
    PickListDropdown.prototype._getTitleText = function (selectedItems) {
        var text = this._getNameFromItem(selectedItems[0]);
        if (selectedItems.length > 1) {
            text = text + " (+" + (selectedItems.length - 1) + ")";
        }
        return text;
    };
    PickListDropdown.prototype.getIndicators = function (selectedItems) {
        var selectedItemIndicators = this.props.selectedItemIndicators;
        if (selectedItems && selectedItems.length && selectedItemIndicators && selectedItemIndicators.length) {
            return React.createElement(ItemIndicatorSet, { indicators: selectedItemIndicators.map(function (indicator) { return indicator.getItemIndicator(selectedItems[0]); }) });
        }
        else {
            return null;
        }
    };
    PickListDropdown.prototype._togglePickList = function (isListOpen) {
        var _this = this;
        var pickListWidth = this.props.pickListWidth || this.props.width || (this._dropdownTitle ? this._dropdownTitle.clientWidth : DEFAULT_PICKLIST_WIDTH);
        this.setState({
            isListOpen: isListOpen,
            pickListWidth: pickListWidth
        }, function () { return _this.props.onToggleDropdown && _this.props.onToggleDropdown(_this.state.isListOpen); });
    };
    PickListDropdown.prototype._getEffectiveIsListOpen = function () {
        return this.props.isOpen !== undefined ? this.props.isOpen : this.state.isListOpen;
    };
    return PickListDropdown;
}(BaseComponent));
export { PickListDropdown };
