import * as tslib_1 from "tslib";
import "../../FabricInit";
import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./PickList.css";
import "./PickListDropdown.css";
import * as React from "react";
import { warn, css } from "office-ui-fabric-react/lib/Utilities";
import { format } from '../../Core/Util/String';
import { SelectionMode } from "office-ui-fabric-react/lib/Selection";
import { FilterOperatorType } from '../../Utilities/Filter';
import { PickListDropdown } from "../../Components/PickList/PickListDropdown";
import { FilterBarItem } from '../../FilterBarItem';
import * as VSSUIResources from '../../VSS.Resources.VSSUI';
import * as Utils_Accessibility from '../../Core/Util/Accessibility';
import { Button } from '../../Button';
var FILTER_PICKLIST_WIDTH = 256;
var PickListFilterBarItem = /** @class */ (function (_super) {
    tslib_1.__extends(PickListFilterBarItem, _super);
    function PickListFilterBarItem(props) {
        var _this = _super.call(this, props) || this;
        _this._onClearClicked = function () {
            _this.setFilterValue({ value: [], operator: _this.state.operator });
            _this.focus();
            Utils_Accessibility.announce(VSSUIResources.AnnounceFilterCleared);
        };
        _this._onSelectionChanged = function (selection) {
            var preventDefault = false;
            if (_this.props.onSelectionChanged) {
                preventDefault = _this.props.onSelectionChanged(selection);
            }
            _this.setFilterValue({ value: selection.selectedItems, operator: selection.selectedOperatorKey });
            return preventDefault;
        };
        _this.onRenderFooter = function () {
            var _a = _this.props, hideClearButton = _a.hideClearButton, selectionMode = _a.selectionMode;
            // Don't show the footer if they explicitly passed true for hideClearButton,
            // or if they have passed undefiend and are in single selection mode.
            if (hideClearButton === true || (hideClearButton === undefined && selectionMode === SelectionMode.single)) {
                return undefined;
            }
            return (React.createElement("div", { className: "vss-PickList--filterBarFooter flex-column" },
                React.createElement(Button, { className: "flex-self-end vss-PickList--filterBarFooterClear", iconProps: { iconName: "Clear" }, subtle: true, disabled: !_this.state.value || _this.state.value.length === 0, ariaLabel: VSSUIResources.PickListClearActionAriaLabel, onClick: _this._onClearClicked }, VSSUIResources.PickListClearActionText)));
        };
        _this._validateProps(props);
        return _this;
    }
    PickListFilterBarItem.prototype.componentWillReceiveProps = function (nextProps) {
        _super.prototype.componentWillReceiveProps.call(this, nextProps);
        this._validateProps(nextProps);
    };
    PickListFilterBarItem.prototype.focus = function () {
        this._pickListDropdown.focus();
    };
    PickListFilterBarItem.prototype.forceUpdate = function () {
        if (this._pickListDropdown) {
            this._pickListDropdown.forceUpdate();
        }
    };
    PickListFilterBarItem.prototype.render = function () {
        var _this = this;
        var _a = this.props, placeholder = _a.placeholder, className = _a.className, pickListClassName = _a.pickListClassName;
        var value = this.state.value;
        var formattedAriaLabel = null;
        var operators = this.props.operators;
        var fullClassName = css("vss-PickListDropdown--filterBar", className);
        var fullPickListClassName = css("vss-PickList--filterBar", pickListClassName);
        if (this.props.showOrAndOperators) {
            operators = [
                { name: VSSUIResources.FilterOperatorOr, key: FilterOperatorType.or },
                { name: VSSUIResources.FilterOperatorAnd, key: FilterOperatorType.and }
            ];
        }
        if (placeholder) {
            if (value && value.length > 0) {
                formattedAriaLabel = format(VSSUIResources.FilterDropdownLabel, placeholder) + " {0}";
            }
            else {
                formattedAriaLabel = format(VSSUIResources.FilterDropdownLabelNoSelection, placeholder);
            }
        }
        return (React.createElement(PickListDropdown, tslib_1.__assign({ className: fullClassName, pickListClassName: fullPickListClassName, componentRef: function (pickListDropdown) { return (_this._pickListDropdown = pickListDropdown); }, showSelectAll: this.props.selectionMode === SelectionMode.multiple, ariaLabelFormat: formattedAriaLabel }, this.props, { operators: operators, selectedItems: value, selectedOperatorKey: this.state.operator, onSelectionChanged: this._onSelectionChanged, pickListWidth: FILTER_PICKLIST_WIDTH, directionalHint: 6 /* bottomRightEdge */, width: undefined, onRenderFooter: this.onRenderFooter })));
    };
    PickListFilterBarItem.prototype._validateProps = function (props) {
        if ($DEBUG) {
            var width = props.width;
            if (width !== undefined) {
                warn("width should not be specified for picklist filter bar item, value will be ignored");
            }
        }
    };
    return PickListFilterBarItem;
}(FilterBarItem));
export { PickListFilterBarItem };
