import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./Table.css";
import * as React from "react";
import { IIconProps } from '../../Icon';
import { IOverlayRenderProps } from '../../List';
import { IBehavior } from '../../Utilities/Behavior';
import { IEventDispatch } from '../../Utilities/Dispatch';
import { ISimpleTableCell, ITable, ITableHeaderCellProps, ITableProps, ITableRowProps, ITableColumn, ITableColumnBehaviorProps, TableColumnLayout } from "./Table.Props";
/**
 * ColumnFill is used to fill the remaining space in the parent element with an
 * empty column. This column can be used anywhere in the column order. Columns
 * that appear after this will be pushed to the right.
 */
export declare const ColumnFill: ITableColumn<{}>;
/**
 * ITableState is the internal state managed by the Table.
 */
export interface ITableState<T> {
    columnBehaviors: Array<IBehavior<Partial<ITableColumnBehaviorProps<T>>, {}>>;
    eventDispatch: IEventDispatch;
    maxWidth?: string;
    minWidth?: string;
    tableBehaviors: Array<IBehavior<Partial<ITableProps<T>>, Partial<ITable<T>>>>;
    tableWidth: string;
}
/**
 * The Table is a multi-column List component with an optional header.
 */
export declare class Table<T> extends React.Component<ITableProps<T>, ITableState<T>> implements ITable<T> {
    static defaultProps: Partial<ITableProps<any>>;
    static getDerivedStateFromProps<T>(props: Readonly<ITableProps<T>>, state: Readonly<ITableState<T>>): Partial<ITableState<T>>;
    private list;
    constructor(props: Readonly<ITableProps<T>>);
    render(): JSX.Element;
    componentDidMount(): void;
    componentDidUpdate(): void;
    componentWillUnmount(): void;
    addOverlay(id: string, rowIndex: number, render: (props: IOverlayRenderProps) => React.ReactNode, zIndex?: number): void;
    removeOverlay(id: string): void;
    getFocusIndex(): number;
    private renderHeader;
    private renderLoadingRow;
    private renderRow;
}
export declare class TableHeaderCell<T> extends React.Component<ITableHeaderCellProps<T>> {
    render(): JSX.Element;
    private onSize;
}
export declare class TableRow<T> extends React.Component<ITableRowProps<T>> {
    private rowElement;
    render(): JSX.Element;
    private onFocus;
    private onPostprocessKeyStroke;
}
export declare function TableCell<T>(props: {
    children?: React.ReactNode;
    className?: string;
    colspan?: number;
    columnIndex: number;
    tableColumn?: ITableColumn<T>;
}): JSX.Element;
export declare function SimpleTableCell<T>(props: {
    children?: React.ReactNode;
    className?: string;
    colspan?: number;
    columnIndex: number;
    contentClassName?: string;
    tableColumn?: ITableColumn<T>;
}): JSX.Element;
export declare function TwoLineTableCell<T>(props: {
    className?: string;
    columnIndex: number;
    colspan?: number;
    iconProps?: IIconProps;
    line1: React.ReactNode;
    line2: React.ReactNode;
    tableColumn?: ITableColumn<T>;
}): JSX.Element;
export declare function renderEmptyCell<T>(rowIndex: number, columnIndex: number): JSX.Element;
export declare function renderSimpleCell<T extends ISimpleTableCell>(rowIndex: number, columnIndex: number, tableColumn: ITableColumn<T>, tableItem: T): JSX.Element;
export declare function renderLoadingCell(columnLayout?: TableColumnLayout): JSX.Element | null;
