import * as tslib_1 from "tslib";
import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./Table.css";
import * as React from "react";
import * as Resources from '../../Resources.Widgets';
import { ObservableLike } from '../../Core/Observable';
import { FocusWithin } from '../../FocusWithin';
import { FocusZone, FocusZoneContext, FocusZoneDirection, FocusZoneKeyStroke } from '../../FocusZone';
import { Icon } from '../../Icon';
import { Intersection } from '../../Intersection';
import { List, renderListCell } from '../../List';
import { Observer } from '../../Observer';
import { Orientation, Position, Sizer } from '../../Sizer';
import { EventDispatch } from '../../Utilities/Dispatch';
import { css, KeyCode } from '../../Util';
import { SortOrder, TableColumnLayout, TableColumnStyle } from "./Table.Props";
/**
 * ColumnFill is used to fill the remaining space in the parent element with an
 * empty column. This column can be used anywhere in the column order. Columns
 * that appear after this will be pushed to the right.
 */
export var ColumnFill = {
    columnLayout: TableColumnLayout.none,
    id: "_fill",
    renderCell: function (rowIndex, columnIndex) {
        return (React.createElement("td", { key: "col-fill", "aria-colindex": columnIndex, className: css("bolt-table-cell bolt-list-cell", "col-" + columnIndex), "data-column-index": columnIndex }));
    },
    renderHeaderCell: function (columnIndex, tableColumn) {
        return (React.createElement("td", { key: "col-fill", className: css(tableColumn.headerClassName, "bolt-table-header-cell bolt-table-header-cell-empty", "col-header-" + columnIndex), "data-column-index": columnIndex }));
    },
    width: -100
};
/**
 * The Table is a multi-column List component with an optional header.
 */
var Table = /** @class */ (function (_super) {
    tslib_1.__extends(Table, _super);
    function Table(props) {
        var _this = _super.call(this, props) || this;
        // Reference to the underlying list interface.
        _this.list = React.createRef();
        _this.renderHeader = function () {
            var _a = _this.props, columns = _a.columns, showHeader = _a.showHeader, renderHeader = _a.renderHeader, spacerWidth = _a.spacerWidth;
            return (React.createElement(React.Fragment, null,
                React.createElement("colgroup", null,
                    React.createElement("col", { key: "col-group-left-spacer", style: { width: spacerWidth === 0 ? spacerWidth + "%" : spacerWidth + "px" } }),
                    columns.map(function (column, columnIndex) {
                        return (React.createElement(Observer, { key: "col-group-" + columnIndex, width: column.width }, function (props) { return React.createElement("col", { style: { width: Math.abs(props.width) + (props.width < 0 ? "%" : "px") } }); }));
                    }),
                    React.createElement("col", { key: "col-group-right-spacer", style: { width: spacerWidth === 0 ? spacerWidth + "%" : spacerWidth + "px" } })),
                showHeader ? renderHeader ? renderHeader(columns) : React.createElement(TableHeader, { tableColumns: columns }) : null));
        };
        _this.renderLoadingRow = function (rowIndex, details) {
            var _a = _this.props, columns = _a.columns, renderLoadingRow = _a.renderLoadingRow;
            var rowDetails = {
                ariaBusy: true,
                data: details.data,
                eventDispatch: _this.state.eventDispatch,
                itemProvider: _this.props.itemProvider,
                onFocusItem: details.onFocusItem,
                renderSpacer: _this.props.renderSpacer,
                selection: _this.props.selection,
                singleClickActivation: _this.props.singleClickActivation,
                excludeTabStop: details.excludeTabStop
            };
            // If a custom row loading animation is available use it.
            if (renderLoadingRow) {
                return renderLoadingRow(rowIndex, rowDetails);
            }
            // Return the default row loading animation.
            return (React.createElement(TableRow, { className: "bolt-list-row-loading", details: rowDetails, index: rowIndex }, columns.map(function (column, columnIndex) {
                return SimpleTableCell({ columnIndex: columnIndex, children: renderLoadingCell(column.columnLayout) });
            })));
        };
        _this.renderRow = function (rowIndex, item, details) {
            var rowDetails = {
                eventDispatch: _this.state.eventDispatch,
                data: details.data,
                itemProvider: _this.props.itemProvider,
                onFocusItem: details.onFocusItem,
                renderSpacer: _this.props.renderSpacer,
                selection: _this.props.selection,
                singleClickActivation: _this.props.singleClickActivation,
                excludeTabStop: details.excludeTabStop
            };
            // First determine if the item supplied a custom row rendering function, if not
            // attempt to use the global row rendering function.
            var renderRow = item.renderRow || _this.props.renderRow;
            if (renderRow) {
                return renderRow(rowIndex, item, rowDetails);
            }
            // If no custom row renderer is available use the default row renderer.
            return (React.createElement(TableRow, { details: rowDetails, index: rowIndex }, _this.props.columns.map(function (tableColumn, columnIndex) {
                return tableColumn.renderCell(rowIndex, columnIndex, tableColumn, item);
            })));
        };
        _this.state = {
            columnBehaviors: [],
            tableBehaviors: [],
            eventDispatch: props.eventDispatch || new EventDispatch(),
            tableWidth: ""
        };
        // Initialize any column behaviors.
        for (var columnIndex = 0; columnIndex < props.columns.length; columnIndex++) {
            var tableColumn = props.columns[columnIndex];
            if (tableColumn.behaviors) {
                for (var _i = 0, _a = tableColumn.behaviors; _i < _a.length; _i++) {
                    var behavior = _a[_i];
                    if (behavior && behavior.initialize) {
                        behavior.initialize({ tableProps: props, columnIndex: columnIndex }, {}, _this.state.eventDispatch);
                    }
                }
            }
        }
        // Initialize the supplied behaviors.
        if (props.behaviors) {
            for (var _b = 0, _c = props.behaviors; _b < _c.length; _b++) {
                var behavior = _c[_b];
                if (behavior.initialize) {
                    behavior.initialize(props, _this, _this.state.eventDispatch);
                }
            }
        }
        return _this;
    }
    Table.getDerivedStateFromProps = function (props, state) {
        var tableBehaviors = [];
        var columnBehaviors = [];
        var tableMaxWidth = 0;
        var tableMinWidth = 0;
        var tableWidth = 0;
        var constrain = false;
        var fill = false;
        for (var _i = 0, _a = props.columns; _i < _a.length; _i++) {
            var column = _a[_i];
            var maxWidth = column.maxWidth, minWidth = column.minWidth;
            var width = ObservableLike.getValue(column.width);
            if (width < 0) {
                // This is a variable width columnn so we will fill the container.
                fill = true;
                // Update the min/max width of the table based on the supplied value.
                // We use a minumum width of 100px if one isnt specified.
                tableMinWidth += minWidth ? minWidth : 0;
                // If we have a variable width column, we will constrain the width by the sum
                // of the fixed width columns and the maxWidth of the variable width columns.
                if (maxWidth) {
                    constrain = true;
                    tableMaxWidth += maxWidth;
                }
            }
            else {
                tableWidth += width;
                tableMinWidth += width;
                tableMaxWidth += width;
            }
        }
        // Build the set of behaviors columns have attached to them
        for (var columnIndex = 0; columnIndex < props.columns.length; columnIndex++) {
            var tableColumn = props.columns[columnIndex];
            if (tableColumn.behaviors) {
                for (var _b = 0, _c = tableColumn.behaviors; _b < _c.length; _b++) {
                    var behavior = _c[_b];
                    columnBehaviors.push(behavior);
                }
            }
        }
        // Build the set of behaviors the table has attached to it
        if (props.behaviors) {
            tableBehaviors.splice.apply(tableBehaviors, [tableBehaviors.length, 0].concat(props.behaviors));
        }
        return {
            columnBehaviors: columnBehaviors,
            tableBehaviors: tableBehaviors,
            maxWidth: constrain ? tableMaxWidth + "px" : undefined,
            minWidth: tableMinWidth !== tableWidth ? tableMinWidth + "px" : undefined,
            tableWidth: fill ? "100%" : tableWidth + "px"
        };
    };
    Table.prototype.render = function () {
        var _a = this.props, className = _a.className, columns = _a.columns, focuszoneProps = _a.focuszoneProps, showLines = _a.showLines, id = _a.id, itemProvider = _a.itemProvider, onActivate = _a.onActivate, onSelect = _a.onSelect, pageSize = _a.pageSize, rowHeight = _a.rowHeight, scrollable = _a.scrollable, selection = _a.selection, singleClickActivation = _a.singleClickActivation;
        var _b = this.state, eventDispatch = _b.eventDispatch, maxWidth = _b.maxWidth, minWidth = _b.minWidth, tableWidth = _b.tableWidth;
        var listProps = {
            className: css(className, "bolt-table", showLines && "bolt-table-show-lines"),
            columns: columns.length + 2,
            eventDispatch: eventDispatch,
            focuszoneProps: focuszoneProps,
            id: id,
            itemProvider: itemProvider,
            maxWidth: maxWidth,
            minWidth: minWidth,
            onActivate: onActivate,
            renderHeader: this.renderHeader,
            renderLoadingRow: this.renderLoadingRow,
            renderRow: this.renderRow,
            onSelect: onSelect,
            role: "grid",
            pageSize: pageSize,
            rowHeight: rowHeight,
            selection: selection,
            singleClickActivation: singleClickActivation,
            width: tableWidth
        };
        var table = (React.createElement("div", { className: css("bolt-table-container flex-grow", scrollable && "scroll-auto") },
            React.createElement(List, tslib_1.__assign({}, listProps, { ref: this.list }))));
        if (scrollable) {
            table = React.createElement(Intersection, { rootMargin: window.innerHeight / 2 }, table);
        }
        return table;
    };
    Table.prototype.componentDidMount = function () {
        // Mount any of the attached tableBehaviors.
        for (var _i = 0, _a = this.state.tableBehaviors; _i < _a.length; _i++) {
            var behavior = _a[_i];
            if (behavior.componentDidMount) {
                behavior.componentDidMount(this.props);
            }
        }
        for (var _b = 0, _c = this.state.columnBehaviors; _b < _c.length; _b++) {
            var behavior = _c[_b];
            if (behavior.componentDidMount) {
                behavior.componentDidMount({ tableProps: this.props });
            }
        }
    };
    Table.prototype.componentDidUpdate = function () {
        // Update any of the attached tableBehaviors.
        for (var _i = 0, _a = this.state.tableBehaviors; _i < _a.length; _i++) {
            var behavior = _a[_i];
            if (behavior.componentDidUpdate) {
                behavior.componentDidUpdate(this.props);
            }
        }
        for (var _b = 0, _c = this.state.columnBehaviors; _b < _c.length; _b++) {
            var behavior = _c[_b];
            if (behavior.componentDidUpdate) {
                behavior.componentDidUpdate({ tableProps: this.props });
            }
        }
    };
    Table.prototype.componentWillUnmount = function () {
        // Unmount any of the attached tableBehaviors.
        for (var _i = 0, _a = this.state.tableBehaviors; _i < _a.length; _i++) {
            var behavior = _a[_i];
            if (behavior.componentWillUnmount) {
                behavior.componentWillUnmount();
            }
        }
        for (var _b = 0, _c = this.state.columnBehaviors; _b < _c.length; _b++) {
            var behavior = _c[_b];
            if (behavior.componentDidUpdate) {
                behavior.componentDidUpdate({ tableProps: this.props });
            }
        }
    };
    Table.prototype.addOverlay = function (id, rowIndex, render, zIndex) {
        if (zIndex === void 0) { zIndex = 0; }
        if (this.list.current) {
            return this.list.current.addOverlay(id, rowIndex, render, zIndex);
        }
    };
    Table.prototype.removeOverlay = function (id) {
        if (this.list.current) {
            return this.list.current.removeOverlay(id);
        }
    };
    Table.prototype.getFocusIndex = function () {
        if (this.list.current) {
            return this.list.current.getFocusIndex();
        }
        return -1;
    };
    Table.defaultProps = {
        showHeader: true,
        showLines: true,
        singleClickActivation: true,
        spacerWidth: 8
    };
    return Table;
}(React.Component));
export { Table };
var TableHeader = /** @class */ (function (_super) {
    tslib_1.__extends(TableHeader, _super);
    function TableHeader() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TableHeader.prototype.render = function () {
        var _this = this;
        return (React.createElement(FocusZoneContext.Consumer, null, function (rowContext) {
            return (React.createElement(FocusZone, { direction: FocusZoneDirection.Horizontal },
                React.createElement("thead", null,
                    React.createElement(FocusWithin, null, function (focusStatus) {
                        return (React.createElement("tr", { className: css("bolt-table-header-row", focusStatus.hasFocus && "focused"), "data-row-index": -1, onBlur: focusStatus.onBlur, onFocus: focusStatus.onFocus },
                            React.createElement("td", { key: "left-spacer", role: "presentation" }),
                            _this.props.tableColumns.map(function (column, columnIndex) {
                                if (column.renderHeaderCell) {
                                    return column.renderHeaderCell(columnIndex, column, rowContext.focuszoneId);
                                }
                                else if (column.iconProps || column.name) {
                                    return (React.createElement(TableHeaderCell, { key: "col-header-" + columnIndex, column: column, columnIndex: columnIndex, focuszoneId: rowContext.focuszoneId },
                                        React.createElement("div", { className: "bolt-table-header-cell-text text-ellipsis" },
                                            column.iconProps && Icon(column.iconProps),
                                            column.name)));
                                }
                                else {
                                    return React.createElement("td", { key: "col-header-" + columnIndex });
                                }
                            }),
                            React.createElement("td", { key: "right-spacer", role: "presentation" })));
                    }),
                    React.createElement("tr", { className: "bolt-table-header-row", "data-row-index": -1 },
                        React.createElement("td", { key: "left-spacer", className: "bolt-table-header-spacer", role: "presentation" }),
                        _this.props.tableColumns.map(function (column, columnIndex) {
                            return (React.createElement("td", { key: "col-header-" + columnIndex, className: css("bolt-table-header-spacer", "col-header-" + columnIndex), role: "presentation" }));
                        }),
                        React.createElement("td", { key: "right-spacer", className: "bolt-table-header-spacer", role: "presentation" })))));
        }));
    };
    return TableHeader;
}(React.Component));
var TableHeaderCell = /** @class */ (function (_super) {
    tslib_1.__extends(TableHeaderCell, _super);
    function TableHeaderCell() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onSize = function (event, updatedSize) {
            var column = _this.props.column;
            // Ensure we havent had our column definition updated and onSize removed.
            if (column.onSize) {
                column.onSize(event, _this.props.columnIndex, updatedSize, column);
            }
        };
        return _this;
    }
    TableHeaderCell.prototype.render = function () {
        var _this = this;
        var _a = this.props, column = _a.column, columnIndex = _a.columnIndex, focuszoneId = _a.focuszoneId;
        return (React.createElement(FocusZoneContext.Consumer, null, function (cellContext) {
            var actionable = (column.behaviors && column.behaviors.length > 0) || !!column.sortProps;
            var _a = column.sortProps, sortProps = _a === void 0 ? {} : _a;
            var _b = sortProps.ariaLabelAscending, ariaLabelAscending = _b === void 0 ? Resources.SortedAscending : _b, _c = sortProps.ariaLabelDescending, ariaLabelDescending = _c === void 0 ? Resources.SortedDescending : _c;
            return (React.createElement("td", { className: css(column.headerClassName, "bolt-table-header-cell", "col-header-" + columnIndex), "data-column-index": columnIndex, role: "columnheader" },
                React.createElement("div", { className: css("bolt-table-header-cell-content flex-row", actionable && "bolt-table-header-cell-actionable"), "data-focuszone": actionable && css(columnIndex === 0 && focuszoneId, cellContext.focuszoneId), tabIndex: actionable ? (columnIndex === 0 ? 0 : -1) : -1 },
                    column.onSize && (React.createElement(Sizer, { className: "bolt-table-header-sizer", divider: false, maxSize: column.maxWidth, minSize: column.minWidth, onSize: _this.onSize, orientation: Orientation.row, position: Position.near, size: column.width })),
                    _this.props.children,
                    column.sortProps && sortProps.sortOrder !== undefined ? (React.createElement(Icon, { ariaLabel: sortProps.sortOrder === SortOrder.ascending ? ariaLabelAscending : ariaLabelDescending, className: "bolt-table-header-sort-icon", iconName: sortProps.sortOrder === SortOrder.ascending ? "SortUp" : "SortDown" })) : null)));
        }));
    };
    return TableHeaderCell;
}(React.Component));
export { TableHeaderCell };
var TableRow = /** @class */ (function (_super) {
    tslib_1.__extends(TableRow, _super);
    function TableRow() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.rowElement = React.createRef();
        _this.onFocus = function (event) {
            _this.props.details.onFocusItem(_this.props.index, event);
        };
        _this.onPostprocessKeyStroke = function (event) {
            if (!event.defaultPrevented) {
                if (event.which === KeyCode.leftArrow && _this.rowElement.current) {
                    _this.rowElement.current.focus();
                    event.preventDefault();
                }
            }
            return FocusZoneKeyStroke.IgnoreNone;
        };
        return _this;
    }
    TableRow.prototype.render = function () {
        var _this = this;
        var _a = this.props, details = _a.details, index = _a.index;
        var ariaLabel = details.ariaLabel, ariaBusy = details.ariaBusy, ariaPosInSet = details.ariaPosInSet, ariaSetSize = details.ariaSetSize, itemProvider = details.itemProvider, excludeFocusZone = details.excludeFocusZone, renderSpacer = details.renderSpacer, role = details.role, selection = details.selection, singleClickActivation = details.singleClickActivation, _b = details.excludeTabStop, excludeTabStop = _b === void 0 ? true : _b;
        return (React.createElement(FocusWithin, { onFocus: this.onFocus }, function (focusStatus) {
            return (React.createElement(FocusZoneContext.Consumer, null, function (rowContext) {
                return (React.createElement(FocusZone, { direction: FocusZoneDirection.Horizontal, postprocessKeyStroke: _this.onPostprocessKeyStroke },
                    React.createElement("tr", { "aria-busy": ariaBusy, "aria-label": ariaLabel, "aria-posinset": ariaPosInSet === undefined ? index + 1 : ariaPosInSet === null ? undefined : ariaPosInSet, "aria-rowindex": index + 1, "aria-selected": selection && selection.selected(index), "aria-setsize": ariaSetSize === undefined ? itemProvider.length : ariaSetSize === null ? undefined : ariaSetSize, className: css(_this.props.className, "bolt-table-row bolt-list-row", index === 0 && "first-row", focusStatus.hasFocus && "focused", selection && selection.selected(index) && "selected", singleClickActivation && "single-click-activation"), "data-focuszone": excludeFocusZone ? undefined : rowContext.focuszoneId, "data-row-index": _this.props.index, role: role, tabIndex: excludeTabStop ? -1 : 0, onBlur: focusStatus.onBlur, onFocus: focusStatus.onFocus, ref: _this.rowElement },
                        React.createElement("td", { key: "left-spacer", className: "bolt-table-cell-compact bolt-table-cell bolt-list-cell bolt-table-spacer-cell", role: "presentation" }, renderSpacer && renderSpacer(index, true)),
                        _this.props.children,
                        React.createElement("td", { key: "right-spacer", className: "bolt-table-cell-compact bolt-table-cell bolt-list-cell bolt-table-spacer-cell", role: "presentation" }, renderSpacer && renderSpacer(index, false)))));
            }));
        }));
    };
    return TableRow;
}(React.Component));
export { TableRow };
export function TableCell(props) {
    var className = props.className, colspan = props.colspan, columnIndex = props.columnIndex, tableColumn = props.tableColumn;
    return (React.createElement("td", { key: "col-" + columnIndex, "aria-colindex": columnIndex, className: css(className, tableColumn && tableColumn.className, "bolt-table-cell bolt-list-cell"), colSpan: colspan, "data-column-index": columnIndex }, props.children));
}
export function SimpleTableCell(props) {
    var children = React.createElement("div", { className: css(props.contentClassName, "bolt-table-cell-content flex-row flex-center") }, props.children);
    return TableCell({
        children: children,
        className: props.className,
        colspan: props.colspan,
        columnIndex: props.columnIndex,
        tableColumn: props.tableColumn
    });
}
export function TwoLineTableCell(props) {
    var rowClasses = "bolt-table-two-line-cell-item flex-row scroll-hidden";
    var line1 = React.createElement("div", { className: rowClasses }, props.line1);
    var line2 = React.createElement("div", { className: rowClasses }, props.line2);
    var lines = (React.createElement(React.Fragment, null,
        line1,
        line2));
    var children;
    if (props.iconProps) {
        children = (React.createElement("div", { className: css(props.className, "bolt-table-cell-content flex-row flex-center") },
            Icon(tslib_1.__assign({}, props.iconProps, { className: css(props.iconProps.className, "bolt-table-two-line-cell-icon flex-noshrink") })),
            React.createElement("div", { className: "flex-column scroll-hidden" }, lines)));
    }
    else {
        children = React.createElement("div", { className: css(props.className, "bolt-table-cell-content flex-column") }, lines);
    }
    return TableCell({
        children: children,
        colspan: props.colspan,
        columnIndex: props.columnIndex,
        className: "bolt-table-two-line-cell",
        tableColumn: props.tableColumn
    });
}
export function renderEmptyCell(rowIndex, columnIndex) {
    return React.createElement(TableCell, { columnIndex: columnIndex, key: columnIndex });
}
export function renderSimpleCell(rowIndex, columnIndex, tableColumn, tableItem) {
    var columnStyle = tableColumn.columnStyle;
    var tableCell = tableItem[tableColumn.id];
    // Do not include padding if the table cell has an href
    var hasLink = !!(tableCell && typeof tableCell !== "string" && typeof tableCell !== "number" && tableCell.href);
    return (React.createElement(SimpleTableCell, { className: css(columnStyle === TableColumnStyle.Primary && "bolt-table-cell-primary", columnStyle === TableColumnStyle.Secondary && "bolt-table-cell-secondary", columnStyle === TableColumnStyle.Tertiary && "bolt-table-cell-tertiary"), columnIndex: columnIndex, contentClassName: hasLink ? "bolt-table-cell-content-with-link" : undefined, key: columnIndex, tableColumn: tableColumn }, tableCell && renderListCell(tableCell)));
}
function getVariableLength() {
    return Math.random() * 80 + 20 + "%";
}
export function renderLoadingCell(columnLayout) {
    if (columnLayout === TableColumnLayout.singleLine || columnLayout === undefined) {
        return (React.createElement("div", { className: "shimmer shimmer-line", style: { width: getVariableLength() } }, "\u00A0"));
    }
    else if (columnLayout === TableColumnLayout.singleLinePrefix) {
        return (React.createElement(React.Fragment, null,
            React.createElement("div", { className: "shimmer shimmer-circle-small flex-noshrink" }),
            React.createElement("div", { className: "shimmer shimmer-line", style: { width: getVariableLength() } }, "\u00A0")));
    }
    else if (columnLayout === TableColumnLayout.twoLine) {
        return (React.createElement("div", { className: "flex-column flex-grow" },
            React.createElement("div", { className: "bolt-table-two-line-cell-item shimmer shimmer-line", style: { width: getVariableLength() } }, "\u00A0"),
            React.createElement("div", { className: "bolt-table-two-line-cell-item shimmer shimmer-line", style: { width: getVariableLength() } }, "\u00A0")));
    }
    else if (columnLayout === TableColumnLayout.twoLinePrefix) {
        return (React.createElement(React.Fragment, null,
            React.createElement("div", { className: "shimmer shimmer-circle-large flex-noshrink" }),
            React.createElement("div", { className: "flex-column flex-grow" },
                React.createElement("div", { className: "bolt-table-two-line-cell-item shimmer shimmer-line", style: { width: getVariableLength() } }, "\u00A0"),
                React.createElement("div", { className: "bolt-table-two-line-cell-item shimmer shimmer-line", style: { width: getVariableLength() } }, "\u00A0"))));
    }
    return null;
}
