import * as tslib_1 from "tslib";
import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./TextField.css";
import * as React from "react";
import { Observer } from '../../Observer';
import { css, getSafeId } from '../../Util';
import { Icon } from '../../Icon';
import { FocusWithin } from '../../FocusWithin';
var inputId = 1;
var TextField = /** @class */ (function (_super) {
    tslib_1.__extends(TextField, _super);
    function TextField(props) {
        var _this = _super.call(this, props) || this;
        _this.inputElement = React.createRef();
        _this.select = function () {
            if (_this.inputElement.current) {
                _this.inputElement.current.select();
            }
        };
        _this.onInputChange = function (e) {
            var onChange = _this.props.onChange;
            if (onChange) {
                onChange(e, e.target.value);
            }
            _this.adjustInputHeight();
        };
        _this.inputId = "textfield-input-" + inputId++;
        return _this;
    }
    TextField.prototype.componentDidMount = function () {
        this.adjustInputHeight();
    };
    TextField.prototype.focus = function () {
        if (this.inputElement.current) {
            this.inputElement.current.focus();
        }
    };
    TextField.prototype.setSelectionRange = function (start, end, direction) {
        if (this.inputElement.current) {
            this.inputElement.current.setSelectionRange(start, end, direction);
        }
    };
    TextField.prototype.render = function () {
        var _this = this;
        var _a = this.props, ariaDescribedBy = _a.ariaDescribedBy, ariaLabel = _a.ariaLabel, autoAdjustHeight = _a.autoAdjustHeight, autoComplete = _a.autoComplete, autoFocus = _a.autoFocus, className = _a.className, containerClassName = _a.containerClassName, description = _a.description, disabled = _a.disabled, _b = _a.focusTreatment, focusTreatment = _b === void 0 ? true : _b, inputClassName = _a.inputClassName, inputType = _a.inputType, label = _a.label, maxLength = _a.maxLength, multiline = _a.multiline, onBlur = _a.onBlur, onClick = _a.onClick, onFocus = _a.onFocus, onKeyDown = _a.onKeyDown, onKeyPress = _a.onKeyPress, onKeyUp = _a.onKeyUp, placeholder = _a.placeholder, prefixIconProps = _a.prefixIconProps, readOnly = _a.readOnly, required = _a.required, resizable = _a.resizable, role = _a.role, rows = _a.rows, spellCheck = _a.spellCheck, suffixIconProps = _a.suffixIconProps, value = _a.value;
        var input = (React.createElement(FocusWithin, { onFocus: onFocus, onBlur: onBlur }, function (focusStatus) {
            var TagName = multiline ? "textarea" : "input";
            var tagSpecificProps = multiline ? { rows: rows } : { type: inputType, autoComplete: autoComplete ? "on" : "off" };
            return (React.createElement("div", { className: css("flex-column", containerClassName) },
                React.createElement("div", { className: css(!label && className, "bolt-textfield flex-row flex-center", disabled && "disabled", focusTreatment && "focus-treatment", focusStatus.hasFocus && "focused") },
                    prefixIconProps &&
                        Icon(tslib_1.__assign({}, prefixIconProps, { className: css(prefixIconProps.className, "bolt-textfield-icon prefix") })),
                    React.createElement(Observer, { value: value || "" },
                        React.createElement(TagName, tslib_1.__assign({}, tagSpecificProps, { "aria-describedby": getSafeId(ariaDescribedBy), "aria-disabled": disabled, "aria-label": ariaLabel, autoFocus: autoFocus, disabled: disabled, className: css(inputClassName, "bolt-textfield-input flex-grow", autoAdjustHeight && "bolt-textfield-auto-adjust", resizable && "bolt-textfield-auto-unresizable"), id: getSafeId(_this.inputId), maxLength: maxLength, onBlur: focusStatus.onBlur, onClick: onClick, onChange: _this.onInputChange, onFocus: focusStatus.onFocus, onKeyDown: onKeyDown, onKeyPress: onKeyPress, onKeyUp: onKeyUp, placeholder: placeholder, readOnly: readOnly, required: required, ref: _this.inputElement, role: role, spellCheck: spellCheck }))),
                    suffixIconProps &&
                        Icon(tslib_1.__assign({}, suffixIconProps, { className: css(suffixIconProps.className, "bolt-textfield-icon suffix") }))),
                description));
        }));
        if (label) {
            return (React.createElement("div", { className: css(className, "flex-column") },
                React.createElement("label", { htmlFor: getSafeId(this.inputId), className: "bolt-textfield-label" }, label),
                input));
        }
        else {
            return input;
        }
    };
    TextField.prototype.adjustInputHeight = function () {
        if (this.props.multiline && this.props.autoAdjustHeight && this.inputElement.current) {
            this.inputElement.current.style.height = "";
            this.inputElement.current.style.height = this.inputElement.current.scrollHeight + "px";
        }
    };
    return TextField;
}(React.Component));
export { TextField };
