/**
 * Class to provide identity information to the VssPersona component.
 *
 * This interface is still in flux and should not be implemented outside of VssPreview.
 */
export interface IIdentityDetailsProvider {
    /**
     * Return a URL to a profile image suitable for display at size pixels.
     */
    getIdentityImageUrl(size: number): string | undefined;
    /**
     * Gets the user's displayname.
     */
    getDisplayName(): string | undefined;
    /**
     * (optional) Render an element to display on delayed hover of the VssPersona
     * e.g. a profile card or tooltip containing the full display name
     */
    onRenderPersonaCard?(image: HTMLElement, onDismiss: () => void): JSX.Element;
}

/**
 * Preset sizes for VssPersonas
 */
export type VssPersonaSize =
    | "extra-extra-small" // 16px
    | "extra-small" // 18px
    | "extra-small-plus" // 20px
    | "small" // 24px
    | "small-plus" // 28px
    | "medium" // 32px
    | "medium-plus" // 40px;
    | "large" // 48px
    | "extra-large" // 72px
    | "extra-extra-large"; // 100px

/**
 * Props for VssPersona
 */
export interface IVssPersonaProps {
    className?: string;
    cssClass?: string;
    /** Size of image to display. Use VssPersonaSize presets. */
    size?: VssPersonaSize;
    /** Provides information about an identity. */
    identityDetailsProvider?: IIdentityDetailsProvider;
    /** If true, do not show persona card on hover */
    suppressPersonaCard?: boolean;
    /** Set when VssPersona is within a focus zone **/
    dataIsFocusable?: boolean;
    /** Set when VssPersona is a tab stop but not in focus zone **/
    isTabStop?: boolean;
    /** Alternate text for VSSPersona image to display when image load does not succeed **/
    imgAltText?: string;
    /** Optional callback on image load error **/
    onImageError?: (event: React.SyntheticEvent<HTMLImageElement>) => void;
    /** Optional, if provided the image url for the Icon. Used if identityDetailsProvider is not set **/
    imageUrl?: string;
    /** Optional, if provided the name to generate the tooltip and initials to use for the persona coin. Used if identityDetailsProvider is not set **/
    displayName?: string;
}