import { IObservableArrayEventArgs, IReadonlyObservableValue, ObservableArrayAction, ObservableValue } from '../Core/Observable';
export interface ISelectionRange {
    beginIndex: number;
    endIndex: number;
}
/**
 * events:
 *  "adjust": - An array ranges that were adjusted.
 *  "select": - The range that contains the udpated selection.
 *  "unselect": - The range that contains the
 */
export interface ISelection extends IReadonlyObservableValue<ISelectionRange[]> {
    multiSelect: boolean;
    selectedCount: number;
    indexSelectable: (index: number) => boolean;
    selected(index: number): boolean;
    clear(): void;
    select(index: number, count?: number, merge?: boolean): void;
    unselect(index: number, count?: number): void;
    lock(): void;
    unlock(): void;
    onItemsChanged: (value: IObservableArrayEventArgs<{}>, action: ObservableArrayAction) => void;
}
export interface ISelectOptions {
    /**
     * Ranges of items whose selection value is locked.  Select and Unselect actions on these items will be ignored.
     */
    lockedRanges?: ISelectionRange[];
    /**
     * Set to true to allow selecting multiple items.
     */
    multiSelect?: boolean;
    /**
     * Initially selected ranges of items.
     */
    selectedRanges?: ISelectionRange[];
}
export interface IHasSelection {
    selection?: ISelection;
}
export interface ISelectableUI {
    /**
     * getFocusIndex will return the row that currently contains focus. If
     * the list doesn't have focus, -1 is returned.
     */
    getFocusIndex(): number;
}
export interface IIndexed {
    index: number;
}
export declare class Selection extends ObservableValue<ISelectionRange[]> implements ISelection {
    lockedRanges: ISelectionRange[];
    multiSelect: boolean;
    selectedCount: number;
    private selectedRanges;
    private lockCount;
    constructor(options?: boolean | ISelectOptions);
    readonly value: ISelectionRange[];
    clear(): void;
    onItemsChanged: (change: IObservableArrayEventArgs<{}>) => void;
    indexSelectable(index: number): boolean;
    selected(index: number): boolean;
    select(index: number, count?: number, merge?: boolean): void;
    unselect(index: number, count?: number): void;
    lock(): void;
    unlock(): void;
}
export declare function indexWithinRanges(index: number, ranges?: ISelectionRange[]): boolean;
