import * as tslib_1 from "tslib";
import { ObservableValue } from '../Core/Observable';
var Selection = /** @class */ (function (_super) {
    tslib_1.__extends(Selection, _super);
    function Selection(options) {
        var _this = _super.call(this, []) || this;
        _this.lockedRanges = [];
        _this.selectedCount = 0;
        _this.selectedRanges = [];
        _this.lockCount = 0;
        _this.onItemsChanged = function (change) {
            var index = change.index;
            // Unselect any items that were removed from the underlying item collection.
            if (change.removedItems && change.removedItems.length) {
                _this.unselect(index, change.removedItems.length);
            }
            // Offset any selection by the items added.
            if (change.addedItems || change.removedItems) {
                var adjustedRanges = [];
                var adjustCount = (change.addedItems ? change.addedItems.length : 0) - (change.removedItems ? change.removedItems.length : 0);
                for (var rangeIndex = 0; rangeIndex < _this.selectedRanges.length; rangeIndex++) {
                    var selectionRange = _this.selectedRanges[rangeIndex];
                    // If the added items are before the range shift it down.
                    if (index <= selectionRange.beginIndex) {
                        // If this adjustment will create a continuous range with the previous range
                        // we merge the ranges.
                        if (rangeIndex > 0 && selectionRange.beginIndex + adjustCount === _this.selectedRanges[rangeIndex - 1].endIndex + 1) {
                            _this.selectedRanges[rangeIndex - 1].endIndex = selectionRange.endIndex + adjustCount;
                            _this.selectedRanges.splice(rangeIndex--, 1);
                            adjustedRanges.push(_this.selectedRanges[rangeIndex]);
                        }
                        else {
                            selectionRange.beginIndex += adjustCount;
                            selectionRange.endIndex += adjustCount;
                            adjustedRanges.push(selectionRange);
                        }
                    }
                    else if (index > selectionRange.beginIndex && index <= selectionRange.endIndex) {
                        // Create the new split selection range.
                        var splitRange = {
                            beginIndex: index + adjustCount,
                            endIndex: selectionRange.endIndex + adjustCount
                        };
                        _this.selectedRanges.splice(++rangeIndex, 0, splitRange);
                        adjustedRanges.push(splitRange);
                        // If the added items are in the middle of selectionRange we need to split the range.
                        selectionRange.endIndex = index - 1;
                        adjustedRanges.push(selectionRange);
                    }
                }
                if (adjustedRanges.length) {
                    _this.notify(adjustedRanges, "adjust");
                }
            }
        };
        if (typeof options === "boolean" || options === undefined) {
            _this.multiSelect = !!options || false;
        }
        else {
            _this.multiSelect = !!options.multiSelect;
            _this.lockedRanges = options.lockedRanges || [];
            _this.selectedRanges = options.selectedRanges || [];
        }
        return _this;
    }
    Object.defineProperty(Selection.prototype, "value", {
        get: function () {
            return this.selectedRanges;
        },
        enumerable: true,
        configurable: true
    });
    Selection.prototype.clear = function () {
        if (!this.lockCount) {
            // Save the current selection ranges for notification.
            var selectedRanges = this.selectedRanges;
            // Reset the selection to an empty selection.
            this.selectedRanges = [];
            this.selectedCount = 0;
            // Go through and notify any observers of the change.
            this.notify(selectedRanges, "unselect");
        }
    };
    Selection.prototype.indexSelectable = function (index) {
        return !indexWithinRanges(index, this.lockedRanges);
    };
    Selection.prototype.selected = function (index) {
        return indexWithinRanges(index, this.selectedRanges);
    };
    Selection.prototype.select = function (index, count, merge) {
        if (!this.lockCount) {
            var updatedRange = void 0;
            if (!this.multiSelect) {
                if (!this.selected(index) && !indexWithinRanges(index, this.lockedRanges)) {
                    this.clear();
                    updatedRange = { beginIndex: index, endIndex: index };
                    this.selectedRanges.push(updatedRange);
                    this.selectedCount++;
                }
            }
            else {
                if (!merge) {
                    this.clear();
                }
                // If no count is specified we will use a single item selection.
                count = count || 1;
                // @TODO: Implement a more optimal multi-count selection
                for (; count > 0; count--) {
                    if (indexWithinRanges(index, this.lockedRanges)) {
                        index++;
                        continue;
                    }
                    var rangeIndex = 0;
                    // Determine if there is a range we can add this selection too.
                    for (; rangeIndex < this.selectedRanges.length; rangeIndex++) {
                        var selectionRange = this.selectedRanges[rangeIndex];
                        // Check if this selection occurs before this selectionRange.
                        if (index < selectionRange.beginIndex) {
                            if (index === selectionRange.beginIndex - 1) {
                                updatedRange = selectionRange;
                                updatedRange.beginIndex--;
                            }
                            break;
                        }
                        // If this index is directly after this range we will extend it.
                        else if (index === selectionRange.endIndex + 1) {
                            // If there is a gap of 1 number we will merge the two ranges.
                            if (rangeIndex < this.selectedRanges.length - 1 && index === this.selectedRanges[rangeIndex + 1].beginIndex - 1) {
                                updatedRange = selectionRange;
                                updatedRange.endIndex = this.selectedRanges[rangeIndex + 1].endIndex;
                                // Remove the second range since it is merged into the previous range.
                                this.selectedRanges.splice(rangeIndex + 1, 1);
                            }
                            else {
                                updatedRange = selectionRange;
                                updatedRange.endIndex++;
                            }
                            break;
                        }
                    }
                    // If there was no range to merge with, add a new one.
                    if (!updatedRange) {
                        updatedRange = { beginIndex: index, endIndex: index };
                        this.selectedRanges.splice(rangeIndex, 0, updatedRange);
                    }
                    this.selectedCount++;
                    index++;
                }
            }
            // Notify observers of the added selection.
            if (updatedRange) {
                this.notify([updatedRange], "select");
            }
        }
    };
    Selection.prototype.unselect = function (index, count) {
        if (!this.lockCount) {
            var updatedRange = void 0;
            // If no count is specified we will use a single item selection.
            count = count || 1;
            // @TODO: Implement a more optimal multi-count selection
            for (; count > 0; count--) {
                if (indexWithinRanges(index, this.lockedRanges)) {
                    index++;
                    continue;
                }
                // Determine the range we are unselecting the item from.
                for (var rangeIndex = 0; rangeIndex < this.selectedRanges.length; rangeIndex++) {
                    var selectionRange = this.selectedRanges[rangeIndex];
                    // If this index if before this range move on to the next one.
                    if (index < selectionRange.beginIndex) {
                        continue;
                    }
                    // Determine whether or not this index falls into this range.
                    if (index >= selectionRange.beginIndex && index <= selectionRange.endIndex) {
                        // If the index is on the start or end of the range, we will just shrink it.
                        // Otherwise we will have to split it.
                        if (index === selectionRange.beginIndex) {
                            selectionRange.beginIndex++;
                        }
                        else if (index === selectionRange.endIndex) {
                            selectionRange.endIndex--;
                        }
                        else {
                            this.selectedRanges.splice(rangeIndex + 1, 0, {
                                beginIndex: index + 1,
                                endIndex: selectionRange.endIndex
                            });
                            selectionRange.endIndex = index - 1;
                        }
                        // Shrinking may have created an empty range, we need to remove it.
                        if (selectionRange.beginIndex > selectionRange.endIndex) {
                            this.selectedRanges.splice(rangeIndex, 1);
                        }
                        this.selectedCount--;
                        // Create a range we use to notify the observers.
                        updatedRange = { beginIndex: index, endIndex: index };
                        break;
                    }
                }
                index++;
            }
            if (updatedRange) {
                this.notify([updatedRange], "unselect");
            }
        }
    };
    Selection.prototype.lock = function () {
        this.lockCount++;
    };
    Selection.prototype.unlock = function () {
        this.lockCount--;
    };
    return Selection;
}(ObservableValue));
export { Selection };
export function indexWithinRanges(index, ranges) {
    if (ranges) {
        for (var _i = 0, ranges_1 = ranges; _i < ranges_1.length; _i++) {
            var range = ranges_1[_i];
            if (index >= range.beginIndex && index <= range.endIndex) {
                return true;
            }
        }
    }
    return false;
}
